@@ -1,5 +1,5 @@
 /*
- * Copyright: (C) 2003 Bruce W. Forsberg
+ * Copyright: (C) 2009 Bruce W. Forsberg
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -15,7 +15,7 @@
  *   License along with this library; if not, write to the Free Software
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
  *
- *   Bruce Forsberg  forsberg@tns.net
+ *   Bruce Forsberg  bruce.forsberg@gmail.com
  *
  */
 
@@ -24,29 +24,17 @@
 #include "otrplayerfile.h"
 #include "id3tag.h"
 
-#include <QKeyEvent>
-#include <qapplication.h>
-#include <qlabel.h>
-#include <qslider.h>
-#include <qdir.h>
-#include <qpalette.h>
-#include <q3listview.h>
-#include <q3multilineedit.h>
-#include <qtimer.h>
-#include <qtextstream.h>
-#include <qmessagebox.h>
-#include <qcombobox.h>
-#include <qlineedit.h>
-#include <qcheckbox.h>
-#include <qspinbox.h>
-#include <qradiobutton.h>
-#include <q3buttongroup.h>
+#include <QtGui>
+#include <Qt3Support>
+
+#include "MyQ3ListViewItem.h"
 
 #include "aflib/aflibConfig.h"
 #include "aflib/aflibFile.h"
+#include "aflib/aflibData.h"
 #include "aflib/aflibAudioFile.h"
 #include "aflib/aflibAudioPitch.h"
-
+#include "aflib/aflibAudioSampleRateCvt.h"
 
 #include <unistd.h>
 #include <sys/types.h>
@@ -63,7 +51,7 @@
 using std::list;
 
 
-#define PLAY_DELTA 512
+#define PLAY_DELTA 10240
 #define MP3_DATA_CFG "otrplayer.cfg"
 #define FILE_LINE 0
 #define INFO_LINE 1
@@ -82,6 +70,8 @@ using std::list;
 otrplayer::otrplayer(QApplication *app, bool debug)
     : mainForm()
 {
+   bool result;
+
    _debug = debug;
 
    if (_debug)
@@ -91,13 +81,33 @@ otrplayer::otrplayer(QApplication *app, bool debug)
    _input = NULL;
    _output = NULL;
    _pitch = NULL;
+   _sample_rate = NULL;
    _stop = TRUE;
    _current_position = 0;
+   _source_current_position = 0;
    _channels = 0;
    _pitch_enabled = FALSE;
    _item = NULL;
    _mount_while_play = FALSE;
    _file_was_playing_when_unmounted = FALSE;
+   _sample_rate_conversion_factor = 1.0;
+
+#ifdef MAEMO
+   aboutAct = new QAction(tr("&About"), this);
+   aboutAct->setStatusTip(tr("Show the application's About box"));
+   result = connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));
+   if (result == false)
+   {
+      if (_debug)
+         std::cout << "Could not connect About menu" << std::endl;
+   }
+
+   helpMenu = menuBar()->addMenu(tr("&Help"));
+   helpMenu->addAction(aboutAct);
+#endif
+
+   // Show sort indicator
+   playList->setShowSortIndicator(true);
 
    if (_debug)
       std::cout << "otrplayer: creating otrplayerfile" << std::endl;
@@ -128,18 +138,6 @@ otrplayer::otrplayer(QApplication *app, bool debug)
       std::cout << "otrplayer: creating timer" << std::endl;
    _timer = new QTimer(this);
    connect(_timer, SIGNAL(timeout()), this, SLOT(run()));
-
-#if 0
-   if (_debug)
-      std::cout << "otrplayer: calling grabKeyboard" << std::endl;
-   QPEApplication::grabKeyboard();
-
-   if (_debug)
-      std::cout << "otrplayer: creating QCopChannel" << std::endl;
-   QCopChannel* pcmciaChannel = new QCopChannel( "QPE/Card", this );
-   connect( pcmciaChannel, SIGNAL(received(const QCString &, const QByteArray &)),
-     this, SLOT(pcmciaMessage(const QCString &, const QByteArray &)) );
-#endif
 }
 
 // Destructor
@@ -156,8 +154,21 @@ otrplayer::~otrplayer()
    delete _input;
    delete _output;
    delete _pitch;
+   delete _sample_rate;
+}
 
-   // QPEApplication::ungrabKeyboard();
+
+void
+otrplayer::about()
+{
+   if (_debug)
+      std::cout << "otrplayer: about called" << std::endl;
+   QMessageBox::about(this, tr("About Otrplayer"),
+      tr("<img src=\"/usr/share/icons/hicolor/scalable/hildon/otrplayer.png\" align=\"middle\" />"
+      "<h1 align=\"center\"><b>Otrplayer 0.3</b></h1>"
+      "<h5 align=\"center\">OTR Audio Player</h5>"
+      "<h5 align=\"center\">(C) 2009 Bruce Forsberg</h5>"
+      "<h5 align=\"center\">http://otrplayer.garage.maemo.org</h5>"));
 }
 
 
@@ -280,6 +291,7 @@ otrplayer::setup(QString& file)
    delete _input;
    delete _output;
    delete _pitch;
+   delete _sample_rate;
 
    if (_debug)
       std::cout << "setup: create aflibAudioFile" << std::endl;
@@ -294,16 +306,31 @@ otrplayer::setup(QString& file)
 
    if (status == AFLIB_SUCCESS)
    {
+       _sample_rate_conversion_factor = 44100.0 / input_config.getSamplesPerSecond();
+
        if (_debug)
           std::cout << "setup: create aflibAudioPitch" << std::endl;
-      _pitch = new aflibAudioPitch(*_input, 1.0);
-      _pitch->enable(_pitch_enabled);
+      _pitch = new aflibAudioPitch(*_input, _sample_rate_conversion_factor);
+      // IF close enough to 1.0 then only enable pitch control is user requested
+      if (_sample_rate_conversion_factor < 1.0000001 && _sample_rate_conversion_factor > 0.9999999)
+         _pitch->enable(_pitch_enabled);
+      // ELSE we need pitch control to sample convert
+      else
+         _pitch->enable(TRUE);
+      _pitch->setOutputSampleRate(TRUE, 44100);
       _pitch->setCacheEnable(FALSE);
       pitchCallback(pitchSlider->value());
 
+#if 0
+      if (_debug)
+         std::cout << "setup: create aflibAudioSampleRateCvt" << std::endl;
+      _sample_rate = new aflibAudioSampleRateCvt(*_input, _sample_rate_conversion_factor);
+      _sample_rate->setCacheEnable(FALSE);
+#endif 
+
        if (_debug)
-          std::cout << "setup: create output aflibAudioFile" << std::endl;
-      aflibConfig output_config(_input->getInputConfig());
+          std::cout << "setup: create output aflibAudioFile with rate " << _sample_rate->getOutputConfig().getSamplesPerSecond() << std::endl;
+      aflibConfig output_config(_pitch->getOutputConfig());
       _output = new aflibAudioFile(*_pitch, AFLIB_DEV_TYPE, "/dev/dsp",
             &output_config, &status);
       _output->setCacheEnable(FALSE);
@@ -318,6 +345,7 @@ otrplayer::setup(QString& file)
       _input = NULL;
       _output = NULL;
       _pitch = NULL;
+      _sample_rate = NULL;
       _stop = TRUE;
       QMessageBox::information(this, "Warning",
          "File is not currently available.\n" + file, QMessageBox::Ok);
@@ -332,7 +360,7 @@ otrplayer::getFiles()
 {
    QStringList  lst, tree;
    QStringList::Iterator  it, itt;
-   Q3ListViewItem  *item = NULL;
+   MyQ3ListViewItem  *item = NULL;
    aflibAudioFile  *temp = NULL;
    aflibConfig  input_config;
    aflibStatus  status;
@@ -354,7 +382,7 @@ otrplayer::getFiles()
       std::cout << "getFiles: Entering getFiles" << std::endl;
 
    // IF there is a currently selected item then save it.
-   item = playList->currentItem();
+   item = (MyQ3ListViewItem *)playList->currentItem();
    if (item != 0)
    {
       dir_save = item->text(DIR_FIELD);
@@ -501,7 +529,7 @@ otrplayer::getFiles()
             if (_otr_file->getOffset(*it) == -1)
                sec = total_sec;
             else
-               sec = _otr_file->getOffset(*it)/input_config.getSamplesPerSecond();
+               sec = _otr_file->getOffset(*it) / input_config.getSamplesPerSecond();
          }
          else
             sec = 0;
@@ -516,7 +544,7 @@ otrplayer::getFiles()
          QChar chr('/');
          int slash;
          slash = (*it).lastIndexOf(chr);
-         item = new Q3ListViewItem( playList, item,
+         item = new MyQ3ListViewItem( playList, item,
             (*it).right((*it).length() - slash - 1),
             str2, str1, str, str3, (*it).left(slash));
       }
@@ -527,7 +555,7 @@ otrplayer::getFiles()
    if (madplay_found == FALSE && displayed_warning == FALSE)
    {
       displayed_warning = TRUE;
-      QMessageBox::information(this, "Warning", "Madplay utility not found.\nPlease install the IPK for madplay.",
+      QMessageBox::information(this, "Warning", "Madplay utility not found.\nPlease install the package for madplay.",
          QMessageBox::Ok);
    }
 
@@ -554,7 +582,7 @@ otrplayer::getFiles()
       // Loop thru entire list
       for ( ; t.current(); t++)
       {
-         Q3ListViewItem  *itm = t.current();
+         MyQ3ListViewItem  *itm = (MyQ3ListViewItem *)t.current();
 
          if (( itm->text(FILE_FIELD) == file_save) &&
               (itm->text(DIR_FIELD) == dir_save))
@@ -579,12 +607,7 @@ otrplayer::play()
    // Call our run routine when app loop has nothing to do
    _timer->start(0);
 
-#if 0
-   // Stop unit from going into suspend mode during playing
-   QCopEnvelope("QPE/System", "setScreenSaverMode(int)" ) << QPEApplication::DisableSuspend;
-#endif
-
-   _item = playList->selectedItem();
+   _item = (MyQ3ListViewItem *)playList->selectedItem();
    if (_item == NULL)
       return;
 
@@ -597,35 +620,37 @@ void
 otrplayer::stop()
 {
    QString  str;
-   Q3ListViewItem  *item;
+   MyQ3ListViewItem  *item;
 
 
    if (_debug)
       std::cout << "stop: stop called" << std::endl;
 
-#if 0
-   // Reenable suspend mode
-   QCopEnvelope("QPE/System", "setScreenSaverMode(int)" ) << QPEApplication::Enable;
-#endif
-
    _timer->stop();
 
    _stop = TRUE;
    delete _input;
    delete _pitch;
    delete _output;
+   delete _sample_rate;
    _pitch = NULL;
    _input = NULL;
    _output = NULL;
+   _sample_rate = NULL;
 
-   item = playList->selectedItem();
+   item = (MyQ3ListViewItem *)playList->selectedItem();
    if (item == NULL)
       return;
 
    str = item->text(DIR_FIELD) + "/" + item->text(FILE_FIELD);
 
    // Update offset of file
-   _otr_file->storeOffset(str, (long)_current_position);
+   if (_current_position == -1)
+      _otr_file->storeOffset(str, (long)_current_position);
+   else
+   {
+      _otr_file->storeOffset(str, (long)_source_current_position );
+   }
 
    // IF a mount or unmount occurred while playing then refresh file display
    if (_mount_while_play == TRUE)
@@ -638,7 +663,8 @@ otrplayer::stop()
 void
 otrplayer::slider(int value)
 {
-   _current_position = _total_size / 100 * value;
+   _current_position = _total_size  * _sample_rate_conversion_factor / 100 * value;
+   _source_current_position = _total_size  * _sample_rate_conversion_factor / 100 * value;
 }
 
 
@@ -646,7 +672,10 @@ void
 otrplayer::pitchCallback(int value)
 {
    if (_pitch)
-      _pitch->setFactor( (double)value/-500.0 + 1.0);
+      if (_pitch_enabled == TRUE)
+         _pitch->setFactor( (double)value/-500.0 + _sample_rate_conversion_factor);
+      else 
+         _pitch->setFactor(_sample_rate_conversion_factor);
 }
 
 
@@ -655,8 +684,18 @@ otrplayer::pitchEnable()
 {
    _pitch_enabled = !_pitch_enabled;
    pitchSlider->setEnabled(_pitch_enabled);
+   // IF close enough to 1.0 then only enable pitch control is user requested
    if (_pitch)
-      _pitch->enable(_pitch_enabled);
+   {
+      if (_sample_rate_conversion_factor < 1.0000001 && _sample_rate_conversion_factor > 0.999999)
+         _pitch->enable(_pitch_enabled);
+      // ELSE we need pitch control to sample convert
+      else
+         _pitch->enable(TRUE);
+   }
+
+   // We need to reset pitch factor is needed so we call this
+   pitchCallback(pitchSlider->sliderPosition());
 }
 
 
@@ -684,7 +723,6 @@ otrplayer::QStringToSeconds(
    int& sec,
    QString  str)
 {
-   int  colon;
    int  min;
    QTextStream ts(&str);
    QChar   dummy;
@@ -700,9 +738,13 @@ void
 otrplayer::seekSeconds(
    int sec)
 {
-   _current_position += (sec * _samples_per_second);
-   if (_current_position < 0)
-      _current_position = 0;
+   _source_current_position += (sec * _samples_per_second / _sample_rate_conversion_factor);
+   if (_source_current_position < 0)
+      _source_current_position = 0;
+   _current_position = _source_current_position;
+
+   stop();
+   play();
 }
 
 
@@ -789,20 +831,23 @@ otrplayer::run()
 
          _output->process(status, _current_position, delta);
 
+         //std::cout << "Position: " << _current_position << " Delta: " << delta << std::endl;
+
          _current_position += delta;
+         _source_current_position += (delta / _sample_rate_conversion_factor);
 
          // Update time, only every second
          if (_samples_per_second == 0)
             sec = 0;
          else
-            sec = _current_position/_samples_per_second;
+            sec = _source_current_position * _sample_rate_conversion_factor /_samples_per_second;
          if (sec != old_sec)
          {
             secondsToQString(sec, str);
             if (_total_size == 0)
                timeSlider->setValue(0);
             else
-               timeSlider->setValue((int)( 100 * _current_position/_total_size));
+               timeSlider->setValue((int)( 100 * _source_current_position * _sample_rate_conversion_factor / _total_size));
             _item->setText(LENGTH_FIELD, str);
          }
          old_sec = sec;
@@ -824,7 +869,7 @@ otrplayer::run()
          if (it.current() != NULL)
          {
             playList->setSelected(it.current(), TRUE);
-            _item = playList->selectedItem();
+            _item = (MyQ3ListViewItem *)playList->selectedItem();
             prepareNextFile();
          }
          else
@@ -862,6 +907,7 @@ otrplayer::prepareNextFile()
    displayID3(_file);
 
    _current_position = _otr_file->getOffset(_file);
+   _source_current_position = _current_position;
 
    // Store this as the last played file
    _otr_file->storePlayedFile(_file);
@@ -871,12 +917,12 @@ otrplayer::prepareNextFile()
 void
 otrplayer::ff()
 {
-   Q3ListViewItem  *item;
+   MyQ3ListViewItem  *item;
    QString         str, str1, str2;
    int             secs;
 
 
-   item = playList->selectedItem();
+   item = (MyQ3ListViewItem *)playList->selectedItem();
    if (item == NULL)
       return;
 
@@ -902,11 +948,11 @@ otrplayer::ff()
 void
 otrplayer::rev()
 {
-   Q3ListViewItem  *item;
+   MyQ3ListViewItem  *item;
    QString         str, str1, str2;
 
 
-   item = playList->selectedItem();
+   item = (MyQ3ListViewItem *)playList->selectedItem();
    if (item == NULL)
       return;
 
@@ -981,19 +1027,6 @@ otrplayer::volumeChange(int value)
    int vol;
    int new_vol;
 
-#if 0
-   // We do this to flush results before QCop call
-   {
-      Config cfg("qpe");
-      cfg.setGroup("Volume");
-      vol = cfg.readNumEntry("VolumePercent", 0);
-
-      vol += value;
-      cfg.writeEntry("VolumePercent",(int)vol);
-   }
-   QCopEnvelope( "QPE/System", "volumeChange(bool)" ) << FALSE;
-#endif
-
    int fd = open("/dev/mixer", O_RDWR);
    ioctl( fd , MIXER_READ(0), &vol );
 
@@ -1051,7 +1084,7 @@ otrplayer::initDirs()
 {
    QStringList  lst;
    QStringList  tree;
-   Q3ListViewItem  *item = NULL;
+   MyQ3ListViewItem  *item = NULL;
    QStringList::Iterator  it, itt;
 
 
@@ -1068,7 +1101,7 @@ otrplayer::initDirs()
 
    for (it = lst.begin(), itt = tree.begin(); it != lst.end(); it++, itt++)
    {
-      item = new Q3ListViewItem( dirView, item, (*it), (*itt));
+      item = new MyQ3ListViewItem( dirView, item, (*it), (*itt));
    }
 }
 
@@ -1103,10 +1136,10 @@ otrplayer::setDir()
 void
 otrplayer::deleteDir()
 {
-   Q3ListViewItem   *item;
+   MyQ3ListViewItem   *item;
 
 
-   item = dirView->selectedItem();
+   item = (MyQ3ListViewItem *)dirView->selectedItem();
    if (item != NULL)
    {
       _otr_file->deleteDir(item->text(0));
@@ -1142,7 +1175,7 @@ otrplayer::removeFiles()
 {
    Q3ListViewItemIterator  it(playList);
    Q3ListViewItemIterator  itt(playList);
-   Q3ListViewItem  *item;
+   MyQ3ListViewItem  *item;
    int   num_items = 0;
    QString  msg_str, num_str;
 
@@ -1154,7 +1187,7 @@ otrplayer::removeFiles()
       long     offset;
       QDir     dir;
 
-      item = it.current();
+      item = (MyQ3ListViewItem *)it.current();
       str = item->text(DIR_FIELD) + "/" + item->text(FILE_FIELD);
 
       // IF offset is -1 then remove
@@ -1181,7 +1214,7 @@ otrplayer::removeFiles()
                long     offset;
                QDir     dir;
 
-               item = itt.current();
+               item = (MyQ3ListViewItem *)itt.current();
                str = item->text(DIR_FIELD) + "/" + item->text(FILE_FIELD);
 
                // IF offset is -1 then remove
@@ -1221,48 +1254,6 @@ otrplayer::fontSize(int size)
 }
 
 
-#if 0
-void
-otrplayer::pcmciaMessage(const QCString &msg, const QByteArray &byt)
-{
-   int i;
-
-
-   if ( msg == "mtabChanged()" )
-   {
-      // IF currently playing then stop while CF ejects.
-      // If one powers down while in play mode the CF card will
-      // eject and this message will be sent. We stop play mode
-      // delay then start again once the unit has been powered
-      // back on. This is a kludge but the only way I kind find
-      // to detect when a CF is unmounted and mounted. If we keep
-      // on playing during power down without doing this then when
-      // we power on madplay will error since the file will not be
-      // available.
-      if (_stop == FALSE)
-      {
-         stop();
-
-         // sleep for 4 seconds
-         for (i =0; i < 4; i++)
-            sleep(1);
-
-         play();
-      }
-      // Need to still delay in case someone presses play right after
-      // power on. This allows for CF card to mount.
-      else
-      {
-         // sleep for 4 seconds
-         for (i =0; i < 4; i++)
-            sleep(1);
-      }
-   }
-}
-
-#endif
-
-
 void
 otrplayer::lookForDecoders()
 {
@@ -1287,4 +1278,3 @@ otrplayer::lookForDecoders()
    }
 }
 
-
